This reverts commit 2f748bf8016c ("datapath: Derive IP protocol...")

This commit is causing some ipv6 fragmentation errors in some older
kernels.  Revert for now and then we can determine how to implement
this patch with appropriate compatability layer changes to prevent
errors on older kernels.

CC: Yi-Hung Wei <[email protected]>
Signed-off-by: Greg Rose <[email protected]>
---
 datapath/flow.c | 22 +++++++++++++---------
 1 file changed, 13 insertions(+), 9 deletions(-)

diff --git a/datapath/flow.c b/datapath/flow.c
index f685cf3..fadc074 100644
--- a/datapath/flow.c
+++ b/datapath/flow.c
@@ -254,18 +254,21 @@ static bool icmphdr_ok(struct sk_buff *skb)
 
 static int parse_ipv6hdr(struct sk_buff *skb, struct sw_flow_key *key)
 {
-       unsigned short frag_off;
-       unsigned int payload_ofs = 0;
        unsigned int nh_ofs = skb_network_offset(skb);
        unsigned int nh_len;
+       int payload_ofs;
        struct ipv6hdr *nh;
-       int err, nexthdr, flags = 0;
+       uint8_t nexthdr;
+       __be16 frag_off;
+       int err;
 
        err = check_header(skb, nh_ofs + sizeof(*nh));
        if (unlikely(err))
                return err;
 
        nh = ipv6_hdr(skb);
+       nexthdr = nh->nexthdr;
+       payload_ofs = (u8 *)(nh + 1) - skb->data;
 
        key->ip.proto = NEXTHDR_NONE;
        key->ip.tos = ipv6_get_dsfield(nh);
@@ -274,9 +277,10 @@ static int parse_ipv6hdr(struct sk_buff *skb, struct 
sw_flow_key *key)
        key->ipv6.addr.src = nh->saddr;
        key->ipv6.addr.dst = nh->daddr;
 
-       nexthdr = ipv6_find_hdr(skb, &payload_ofs, -1, &frag_off, &flags);
-       if (flags & IP6_FH_F_FRAG) {
-               if (frag_off)
+       payload_ofs = ipv6_skip_exthdr(skb, payload_ofs, &nexthdr, &frag_off);
+
+       if (frag_off) {
+               if (frag_off & htons(~0x7))
                        key->ip.frag = OVS_FRAG_TYPE_LATER;
                else
                        key->ip.frag = OVS_FRAG_TYPE_FIRST;
@@ -284,11 +288,11 @@ static int parse_ipv6hdr(struct sk_buff *skb, struct 
sw_flow_key *key)
                key->ip.frag = OVS_FRAG_TYPE_NONE;
        }
 
        /* Delayed handling of error in ipv6_find_hdr() as it
         * always sets flags and frag_off to a valid value which may be
         * used to set key->ip.frag above.
         */
-       if (unlikely(nexthdr < 0))
+       if (unlikely(payload_ofs < 0))
                return -EPROTO;
 
        nh_len = payload_ofs - nh_ofs;
@@ -284,11 +288,11 @@ static int parse_ipv6hdr(struct sk_buff *skb, struct 
        }
 
-       /* Delayed handling of error in ipv6_find_hdr() as it
-        * always sets flags and frag_off to a valid value which may be
+       /* Delayed handling of error in ipv6_skip_exthdr() as it
+        * always sets frag_off to a valid value which may be
         * used to set key->ip.frag above.
         */
-- 
1.8.3.1

_______________________________________________
dev mailing list
[email protected]
https://mail.openvswitch.org/mailman/listinfo/ovs-dev

Reply via email to